module SshMenu
  class Selector

    def initialize(connections)
      @connections = connections
    end

    def select
      selection = request_selection

      case selection
      when /\A[eE]\z/
        {:action => :edit}
      when (/\A[1-9][0-9]*\z/)
        index = Integer(selection) - 1
        raise_invalid unless index.between?(0, @connections.length - 1)
        {:index => index}
      when /\A\z/
        {}
      else
        raise_invalid
      end
    end

    def raise_invalid
      raise ArgumentError, 'selection must be an index from the list'
    end

    def request_selection
      puts connection_list
      print 'Pick a server (or e to edit connections): '
      STDIN.gets.chomp
    end

    def connection_list
      uh_length = @connections.map {|conn| conn['user_host'].length}.max
      index_length = @connections.size.to_s.length

      @connections.each.with_index.each_with_object(StringIO.new) do |(connection, index), concat|
        concat << "%#{index_length}d) %-#{uh_length+3}s %s\n" % [
          index + 1,
          connection['user_host'],
          connection['description']
        ]
      end.string
    end
  end
end
